"use strict";
let flipClockInstance = null;

window.onload = function () {
  const canvas = document.querySelector('canvas');
  const flipclock = document.getElementById('flipclock');

  if (!canvas || !flipclock) {
    console.error('Canvas oder Flipclock-Element nicht gefunden.');
    return;
  }

  // Set canvas size to 150% of viewport size
  canvas.style.height = '150vh';
  canvas.style.width = '150vw';

  // Center the canvas
  canvas.style.position = 'absolute';
  canvas.style.top = '50%';
  canvas.style.left = '50%';
  canvas.style.transform = 'translate(-50%, -50%)';

  // Center the flipclock
  flipclock.style.position = 'absolute';
  flipclock.style.top = '50%';
  flipclock.style.left = '50%';
  flipclock.style.transform = 'translate(-50%, -50%) scale(1.8)';

  // Scale the flipclock elements
  const flipclockElements = flipclock.querySelectorAll('*');
  flipclockElements.forEach(el => {
    el.style.transform = 'scale(1.0)';
  });
};


class Control {
  constructor(elSelector) {
    this.el = document.querySelector(elSelector);
    this.els = Array.from(this.el.querySelectorAll("li"));
    this.curr = 0;
  }

  process(value) {
    const curli = this.els.find(el => el.classList.contains("active"));
    const nextli = curli === this.els[this.els.length - 1] ? this.els[0] : this.els[1];

    this.updateFlipText(nextli, value, true);

    this.els.forEach(el => el.classList.remove("before", "active"));
    curli.classList.add("before");
    nextli.classList.add("active");

    this.curr = nextli === this.els[0] ? 0 : 1;

    setTimeout(() => {
      this.updateFlipText(nextli, value, false);
    }, 250);
  }

  updateFlipText(element, value, isUpper) {
    const formattedValue = FlipClock.addZeros(value, 1);
    const selector = isUpper ? ".flip-up .flip-text" : ".flip-down .flip-text";
    const textElement = element.querySelector(selector);
    if (textElement) {
      textElement.textContent = formattedValue;
    }
  }

  setValue(value) {
    const formattedValue = FlipClock.addZeros(value, 1);
    this.els.forEach(li => {
      this.updateFlipText(li, formattedValue, true);
      this.updateFlipText(li, formattedValue, false);
    });
  }
}

class FlipClock {
  constructor() {
    this.isPaused = true;
    this.saveSec = 0;
    this.saveMin = 0;
    this.saveHou = 0;
    this.Second = null;
    this.Minute = null;
    this.Hour = null;
    this.timeoutCallback = null;
    this.flipClock = document.getElementById("flipclock");
    this.currentAMPM = "AM";
  }

  static addZeros(num, digits) {
    return num.toString().padStart(digits + 1, '0');
  }

  init() {
    this.isPaused = false;
    this.Second = new Control(".flip-second");
    this.Minute = new Control(".flip-minute");
    this.Hour = new Control(".flip-hour");

    this.Hour.originalProcess = this.Hour.process;
    this.Hour.process = (value) => {
      const { displayHours, apm } = this.formatHours(value);
      this.Hour.originalProcess(displayHours);
      this.updateAMPMAndDate(apm);  // Jedes Mal AM/PM und Datum aktualisieren
    };

    this.Hour.originalSetValue = this.Hour.setValue;
    this.Hour.setValue = (value) => {
      const { displayHours, apm } = this.formatHours(value);
      this.Hour.originalSetValue(displayHours);
      this.updateAMPMAndDate(apm);  // Jedes Mal AM/PM und Datum aktualisieren
    };

    const currentTime = this.getTime();
    this.Second.setValue(currentTime.seconds);
    this.Minute.setValue(currentTime.minutes);
    this.Hour.setValue(currentTime.hours);
  }

  formatHours(hours) {
    let displayHours = hours;
    let apm = this.getAMPM(hours);

    if (window._showAPM) {
      if (hours > 12) {
        displayHours = hours - 12;
      } else if (hours === 0) {
        displayHours = 12;
      }
    }

    if (window._showAPM && apm === "AM" && window._hideLeadingZeroAM && displayHours < 10) {
      displayHours = displayHours.toString();
    } else {
      displayHours = FlipClock.addZeros(displayHours, 1);
    }

    return { displayHours, apm };
  }

  getAMPM(hours) {
    return hours >= 12 ? "PM" : "AM";
  }

  updateAMPMAndDate(apm) {
    if (window._showAPM) {
      this.Hour.els.forEach(li => {
        const apmElement = li.querySelector(".flip-apm");
        if (apmElement) {
          apmElement.textContent = apm;
        }
      });
    } else {
      // Sicherstellen, dass AM/PM entfernt wird, wenn es nicht angezeigt werden soll
      this.Hour.els.forEach(li => {
        const apmElement = li.querySelector(".flip-apm");
        if (apmElement) {
          apmElement.textContent = "";  // AM/PM löschen
        }
      });
    }

    if (window._showDate) {
      const options = {
        weekday: "long",
        year: "numeric",
        month: "long",
        day: "numeric"
      };
      const datex = new Date().toLocaleDateString(undefined, options);
      this.Hour.els.forEach(li => {
        const dateElement = li.querySelector(".flip-date");
        if (dateElement) {
          dateElement.textContent = datex;
        }
      });
    } else {
      // Sicherstellen, dass das Datum entfernt wird, wenn es nicht angezeigt werden soll
      this.Hour.els.forEach(li => {
        const dateElement = li.querySelector(".flip-date");
        if (dateElement) {
          dateElement.textContent = "";  // Datum löschen
        }
      });
    }
  }

  getTime() {
    const currentTime = new Date();
    return {
      value: currentTime,
      seconds: currentTime.getSeconds(),
      minutes: currentTime.getMinutes(),
      hours: currentTime.getHours()
    };
  }

  start() {
    this.engine();
  }

  pause() {
    this.isPaused = true;
    clearTimeout(this.timeoutCallback);
  }

  resume() {
    this.isPaused = false;
    this.engine();
  }

  engine() {
    if (this.isPaused) return;

    const time = this.getTime();

    if (this.saveSec !== time.seconds) {
      this.saveSec = time.seconds;
      this.Second.process(time.seconds);
    }

    if (this.saveMin !== time.minutes) {
      this.saveMin = time.minutes;
      this.Minute.process(time.minutes);
    }

    if (this.saveHou !== time.hours) {
      this.saveHou = time.hours;
      this.Hour.process(time.hours);
    }

    this.timeoutCallback = setTimeout(() => this.engine(), 1000 - (Date.now() % 1000));
  }
}

function StartClock() {
  flipClockInstance = new FlipClock();  // Hier global setzen
  flipClockInstance.init();
  flipClockInstance.start();
}

function setParameter(showAPM, showDate, showSec, hideLeadingZeroAM) {
  window._showAPM = showAPM;
  window._showDate = showDate;
  window._showSec = showSec;
  window._hideLeadingZeroAM = hideLeadingZeroAM;

  if (!window._showSec) {
    disableClass(".flip-pane-2", 0);
  } else {
    disableClass(".flip-pane-2", 1);
  }
  StartClock();
}
function setParameterX(showAPM, showDate, showSec, hideLeadingZeroAM) {
  const restartClock = (window._showAPM !== showAPM || window._showDate !== showDate);

  window._showAPM = showAPM;
  window._showDate = showDate;
  window._showSec = showSec;
  window._hideLeadingZeroAM = hideLeadingZeroAM;

  // Restart nur bei AM/PM oder Datum ändern
  updateClockSettings(restartClock);
}
function scaleCanvas(newScale) {
  const flipclock = document.getElementById('flipclock');
  if (flipclock) {
    flipclock.style.transform = `translate(-50%, -50%) scale(${newScale})`;
  } else {
    console.error('Flipclock element not found.');
  }
}
function zoomCanvas(newScale) {
  document.body.style.transform = `scale(${newScale})`;
}

function setBrightness(newBrightness) {
  document.body.style.filter = `brightness(${newBrightness})`;
}

function disableClass(name, opa) {
  const elements = document.querySelectorAll(name);
  const elementsX = document.querySelectorAll(".flip-pane-1");

  elements.forEach(el => el.style.opacity = opa);

  if (opa === 0) {
    elementsX.forEach(el => el.style.margin = "-10vw 0vw 0vw -21.0vw");
  } else if (opa === 1) {
    elementsX.forEach(el => el.style.margin = "-10vw 0vw 0vw -25.5vw");
  }
}
function updateClockSettings() {
  if (window._showSec) {
    disableClass(".flip-pane-2", 1);
  } else {
    disableClass(".flip-pane-2", 0);
  }

  const flipClock = document.getElementById("flipclock");

  if (window._showAPM) {
    flipClock.classList.add('show-apm');
  } else {
    flipClock.classList.remove('show-apm');
  }

  if (window._showDate) {
    flipClock.classList.add('show-date');
  } else {
    flipClock.classList.remove('show-date');
  }

  // Aktualisiere AM/PM und Datum immer korrekt nach den neuen Einstellungen
  const currentTime = new Date();
  const apm = currentTime.getHours() >= 12 ? "PM" : "AM";  // Berechne das aktuelle AM/PM
  flipClockInstance.updateAMPMAndDate(apm);  // Aktualisiere die AM/PM- und Datumsanzeige
}
document.addEventListener("DOMContentLoaded", () => {
  //setParameter(true, true, true, true);
  //StartClock();
});